/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file MulticastSender.C
 *    Implementation of MulticastSender.h.
 *
 * @author Tim Langner
 * @date   2010/07/27
 */

#include <communication/MulticastSender.h>

#include <boost/asio/buffer.hpp>

namespace mira {

///////////////////////////////////////////////////////////////////////////////

MulticastSender::MulticastSender(const std::string& address, uint16 port):
	mEndpoint(boost::asio::ip::address::from_string(address), port),
	mSocket(static_cast<boost::asio::io_service&>(mService), mEndpoint.protocol()),
	mTimer(static_cast<boost::asio::io_service&>(mService))
{
}

MulticastSender::MulticastSender(const std::string& address, uint16 port,
                                 boost::asio::io_service& service) :
	mService(service),
	mEndpoint(boost::asio::ip::address::from_string(address), port),
	mSocket(static_cast<boost::asio::io_service&>(mService), mEndpoint.protocol()),
	mTimer(static_cast<boost::asio::io_service&>(mService))
{
}

void MulticastSender::start(const Duration& interval,
                            const std::string& message)
{
	mInterval = interval;
	mMessage = message;
	try
	{
		mSocket.send_to(boost::asio::buffer(mMessage), mEndpoint);
		mTimer.expires_from_now(mInterval);
		mTimer.async_wait(boost::bind(&MulticastSender::handleTimeout, this,
		                              boost::asio::placeholders::error));
	}
	catch(...)
	{

	}
}

void MulticastSender::send(const std::string& message)
{
	try
	{
		mSocket.send_to(boost::asio::buffer(message), mEndpoint);
	}
	catch(...)
	{

	}
}

void MulticastSender::handleTimeout(const boost::system::error_code& error)
{
	if (!error)
	{
		try
		{
			mSocket.send_to(boost::asio::buffer(mMessage), mEndpoint);
			mTimer.expires_from_now(mInterval);
			mTimer.async_wait(boost::bind(&MulticastSender::handleTimeout, this,
			                              boost::asio::placeholders::error));
		}
		catch(...)
		{

		}
	}
	else
		MIRA_LOG(ERROR) << "An error occurred in MulticastSender code=" << error;
}

///////////////////////////////////////////////////////////////////////////////

} // namespace
