/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file VisualizationPlotView.h
 *    Base class for visualization of svg graphs.
 *
 * @author Erik Einhorn, Ronny Stricker
 * @date   2014/03/12
 */

#ifndef _MIRA_GRAPHVIEW_H_
#define _MIRA_GRAPHVIEW_H_

#include <QDragMoveEvent>
#include <QGraphicsView>

#include <GraphVisualizationExports.h>

class QGraphicsItem;
class QSvgRenderer;

namespace mira {

///////////////////////////////////////////////////////////////////////////////

class MIRA_GRAPHVISUALIZATION_EXPORT GraphView : public QGraphicsView
{
	Q_OBJECT
public:

	GraphView(QWidget* parent);
	~GraphView();

public slots:

	void updateGraph();
	void updateGraph( QString const& graph );
	void saveGraph();
	void showText( QString const& text);
	void setAutoScale( bool autoscale );

protected:

	virtual void mouseMoveEvent(QMouseEvent *event);
	virtual void mousePressEvent(QMouseEvent *event);
	virtual void mouseReleaseEvent(QMouseEvent *event);
	virtual void wheelEvent (QWheelEvent * event);


signals:
	void autoScaleChanged( bool autoscale );

private:

	bool mPan;
	bool mAutoScale;
	int mPanStartX, mPanStartY;

	QString mGraph;

	QGraphicsScene* mScene;
	QGraphicsItem* mGraphItem;
	QGraphicsItem* mTextItem;
	QSvgRenderer* mSvgRenderer;

};

///////////////////////////////////////////////////////////////////////////////

}

#endif 
